@@ -5,10 +5,7 @@
 
 import ConfigParser
 import os
-try:
-    from json import loads
-except ImportError:
-    from simplejson import loads
+from rst2pdf.rson import loads
 
 cfdir = os.path.join(os.path.expanduser('~'), '.rst2pdf')
 cfname = os.path.join(cfdir, 'config')
