@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
-#$HeadURL: https://rst2pdf.googlecode.com/svn/branches/0.14/rst2pdf/createpdf.py $
-#$LastChangedDate: 2010-03-22 23:02:32 -0300 (Mon, 22 Mar 2010) $
-#$LastChangedRevision: 2019 $
+#$HeadURL: https://rst2pdf.googlecode.com/svn/tags/0.16/rst2pdf/createpdf.py $
+#$LastChangedDate: 2010-10-06 18:21:41 -0300 (Wed, 06 Oct 2010) $
+#$LastChangedRevision: 2396 $
 
 # See LICENSE.txt for licensing terms
 
@@ -61,6 +61,8 @@ import docutils.readers.doctree
 import docutils.core
 import docutils.nodes
 from docutils.parsers.rst import directives
+from docutils.readers import standalone
+from docutils.transforms import Transform
 
 import pygments_code_block_directive # code-block directive
 import oddeven_directive 
@@ -74,10 +76,12 @@ from reportlab.pdfbase.pdfdoc import PDFPageLabel
 #from reportlab.lib.pagesizes import *
 
 from flowables import * # our own reportlab flowables
+from sinker import Sinker
 import flowables
-from image import MyImage
+from image import MyImage, missing
 
 from aafigure_directive import Aanode
+import counter_role
 
 from log import log, nodeid
 from pprint import pprint
@@ -86,6 +90,18 @@ from smartypants import smartyPants
 
 from roman import toRoman
 
+# Small template engine for covers
+# The obvious import doesn't work for complicated reasons ;-)
+import tenjin
+to_str=tenjin.helpers.generate_tostrfunc('utf-8')
+escape=tenjin.helpers.escape
+templateEngine=tenjin.Engine()
+
+def renderTemplate(tname, **context):
+  context['to_str']=to_str
+  context['escape']=escape
+  return templateEngine.render(tname, context)
+
 # Is this really the best unescape in the stdlib for '&amp;' => '&'????
 from xml.sax.saxutils import unescape, escape
 
@@ -114,7 +130,8 @@ numberingstyles={ 'arabic': 'ARABIC',
 
 class RstToPdf(object):
 
-    def __init__(self, stylesheets=[], language=None,
+    def __init__(self, stylesheets=[],
+                 language='en',
                  header=None,
                  footer=None,
                  inlinelinks=False,
@@ -122,19 +139,24 @@ class RstToPdf(object):
                  font_path=[],
                  style_path=[],
                  fit_mode='shrink',
+                 background_fit_mode='center',
                  sphinx=False,
                  smarty='0',
                  baseurl=None,
                  repeat_table_rows=False,
                  footnote_backlinks=True,
                  inline_footnotes=False,
+                 real_footnotes=False,
                  def_dpi=300,
                  show_frame=False,
                  highlightlang='python', #This one is only used by sphinx
                  basedir=os.getcwd(),
                  splittables=False,
                  blank_first_page=False,
-                 breakside='odd'
+                 breakside='odd',
+                 custom_cover='cover.tmpl',
+                 floating_images=False,
+                 numbered_links=False,
                  ):
         self.debugLinesPdf=False
         self.depth=0
@@ -143,10 +165,25 @@ class RstToPdf(object):
         self.splittables=splittables
         self.basedir=basedir
         self.language = language
+        try:
+            get_language (self.language)
+        except ImportError:
+            try:
+                language = self.language.split('_', 1)[0]
+                get_language (language)
+                self.language = language
+            except ImportError:
+                log.warning("Can't load Docutils module "\
+                    "for language %s or %s", self.language, language)
+                self.language = 'en'
         self.doc_title = ""
+        self.doc_title_clean = ""
+        self.doc_subtitle = ""
         self.doc_author = ""
         self.header = header
         self.footer = footer
+        self.custom_cover=custom_cover
+        self.floating_images=floating_images
         self.decoration = {'header': header,
                            'footer': footer,
                            'endnotes': [],
@@ -167,14 +204,20 @@ class RstToPdf(object):
         self.inlinelinks = inlinelinks
         self.breaklevel = breaklevel
         self.fit_mode = fit_mode
+        self.background_fit_mode = background_fit_mode
         self.to_unlink = []
         self.smarty = smarty
         self.baseurl = baseurl
         self.repeat_table_rows = repeat_table_rows
         self.footnote_backlinks = footnote_backlinks
         self.inline_footnotes = inline_footnotes
+        self.real_footnotes = real_footnotes
+        # Real footnotes are always a two-pass thing.
+        if self.real_footnotes:
+            self.mustMultiBuild = True
         self.def_dpi = def_dpi
         self.show_frame = show_frame
+        self.numbered_links = numbered_links
         self.img_dir = os.path.join(self.PATH, 'images')
 
         # Sorry about this, but importing sphinx.roles makes some
@@ -192,6 +235,8 @@ class RstToPdf(object):
             import math_directive
             self.gen_pdftext, self.gen_elements = nodehandlers(self)
 
+        self.sphinx = sphinx
+
         if not self.styles.languages:
             self.styles.languages=[]
             if self.language:
@@ -297,6 +342,38 @@ class RstToPdf(object):
             sep = ';'
         return sep + " "
 
+
+    def styleToTags(self, style):
+        '''Takes a style name, returns a pair of opening/closing tags for it, like
+        "<font face=helvetica size=14 color=red>". Used for inline
+        nodes (custom interpreted roles)'''
+
+        try:
+            s = self.styles[style]
+            r1=['<font face="%s" color="#%s" ' %
+                (s.fontName, s.textColor.hexval()[2:])]
+            bc = s.backColor
+            if bc:
+                r1.append('backColor="#%s"' % bc.hexval()[2:])
+            if s.trueFontSize:
+                r1.append('size="%d"'%s.fontSize)
+            r1.append('>')
+            r2=['</font>']
+
+            if s.strike:
+                r1.append('<strike>')
+                r2.insert(0,'</strike>')
+            if s.underline:
+                r1.append('<u>')
+                r2.insert(0,'</u>')
+            
+            return [''.join(r1), ''.join(r2)]
+        except KeyError:
+            log.warning('Unknown class %s', style)
+            return None
+
+        
+
     def styleToFont(self, style):
         '''Takes a style name, returns a font tag for it, like
         "<font face=helvetica size=14 color=red>". Used for inline
@@ -304,15 +381,15 @@ class RstToPdf(object):
 
         try:
             s = self.styles[style]
+            r=['<font face="%s" color="#%s" ' %
+                (s.fontName, s.textColor.hexval()[2:])]
             bc = s.backColor
             if bc:
-                r = '<font face="%s" size="%d" color="#%s" backColor="#%s">'\
-                    %(s.fontName, s.fontSize,
-                      s.textColor.hexval()[2:], bc.hexval()[2:])
-            else:
-                r = '<font face="%s" size="%d" color="#%s">' % (
-                    s.fontName, s.fontSize, s.textColor.hexval()[2:])
-            return r
+                r.append('backColor="#%s"' % bc.hexval()[2:])
+            if s.trueFontSize:
+                r.append('size="%d"'%s.fontSize) 
+            r.append('>')
+            return ''.join(r)
         except KeyError:
             log.warning('Unknown class %s', style)
             return None
@@ -373,7 +450,7 @@ class RstToPdf(object):
         Takes a list of rows, consisting of cells and performs the following fixes:
 
         * For multicolumn cells, add continuation cells, to make all rows the same
-        size.
+        size. These cells have to be multirow if the original cell is multirow.
 
         * For multirow cell, insert continuation cells, to make all columns the
         same size.
@@ -388,6 +465,8 @@ class RstToPdf(object):
         # If there is a multicol cell, we need to insert Continuation Cells
         # to make all rows the same length
 
+        #from pudb import set_trace; set_trace()
+
         for y in range(0, len(rows)):
             for x in range(0, len(rows[y])):
                 cell = rows[y][x]
@@ -395,7 +474,9 @@ class RstToPdf(object):
                     continue
                 if cell.get("morecols"):
                     for i in range(0, cell.get("morecols")):
-                        rows[y].insert(x + 1, "")
+                        e=docutils.nodes.entry("")
+                        e["morerows"] = cell.get("morerows",0)
+                        rows[y].insert(x + 1, e)
 
         for y in range(0, len(rows)):
             for x in range(0, len(rows[y])):
@@ -437,7 +518,7 @@ class RstToPdf(object):
         # Pass a ridiculous size, then it will shrink to what's available
         # in the frame
         return BoundByWidth(2000*cm,
-            content=[XPreformatted(text, style)],
+            content=[XXPreformatted(text, style)],
             mode=self.fit_mode, style=style)
 
     def createPdf(self, text=None, 
@@ -469,7 +550,7 @@ class RstToPdf(object):
         if doctree is None:
             if text is not None:
                 if self.language:
-                    settings_overrides={'language_code': self.language[:2]}
+                    settings_overrides={'language_code': self.language}
                 else:
                     settings_overrides={}
                 self.doctree = docutils.core.publish_doctree(text,
@@ -482,15 +563,52 @@ class RstToPdf(object):
         else:
             self.doctree = doctree
 
+        if self.numbered_links:
+            # Transform all links to sections so they show numbers
+            from sectnumlinks import SectNumFolder, SectRefExpander
+            snf = SectNumFolder(self.doctree)
+            self.doctree.walk(snf)
+            srf = SectRefExpander(self.doctree, snf.sectnums)
+            self.doctree.walk(srf)
+            
         elements = self.gen_elements(self.doctree)
 
+        # Find cover template, save it in cover_file
+        def find_cover(name):
+            cover_path=[self.basedir, os.path.expanduser('~/.rst2pdf'),
+                os.path.join(self.PATH,'templates')]
+            cover_file=None
+            for d in cover_path:
+                if os.path.exists(os.path.join(d,name)):
+                    cover_file=os.path.join(d,name)
+                    break
+            return cover_file
+
+        cover_file=find_cover(self.custom_cover)
+        if cover_file is None:
+            log.error("Can't find cover template %s, using default"%self.custom_cover)
+            cover_file=find_cover('cover.tmpl')
+
+        # Feed data to the template, get restructured text.
+        cover_text = renderTemplate(tname=cover_file,
+                            title=self.doc_title,
+                            subtitle=self.doc_subtitle
+                        )
+
+        # This crashes sphinx because .. class:: in sphinx is
+        # something else. Ergo, pdfbuilder does it in its own way.
+        if not self.sphinx:
+
+            elements = self.gen_elements(
+                publish_secondary_doctree(cover_text, self.doctree, source_path)) + elements
+
         if self.blank_first_page:
             elements.insert(0,PageBreak())
 
         # Put the endnotes at the end ;-)
         endnotes = self.decoration['endnotes']
         if endnotes:
-            elements.append(Spacer(1, 2*cm))
+            elements.append(MySpacer(1, 2*cm))
             elements.append(Separation())
             for n in self.decoration['endnotes']:
                 t_style = TableStyle(self.styles['endnote'].commands)
@@ -498,7 +616,23 @@ class RstToPdf(object):
                 elements.append(DelayedTable([[n[0], n[1]]],
                     style=t_style, colWidths=colWidths))
 
-        
+        if self.floating_images:
+            #from pudb import set_trace; set_trace()
+            # Handle images with alignment more like in HTML
+            new_elem=[]
+            for i,e in enumerate(elements[::-1]):
+                if isinstance (e, MyImage) and e.image.hAlign != 'CENTER'\
+                and new_elem:
+                    # This is an image where flowables should wrap
+                    # around it
+                    popped=new_elem.pop()
+                    new_elem.append(ImageAndFlowables(e,popped,
+                        imageSide=e.image.hAlign.lower()))
+                else:
+                    new_elem.append(e)
+
+            elements = new_elem
+            elements.reverse()
 
         head = self.decoration['header']
         foot = self.decoration['footer']
@@ -506,38 +640,84 @@ class RstToPdf(object):
         # So, now, create the FancyPage with the right sizes and elements
         FP = FancyPage("fancypage", head, foot, self)
 
+        def cleantags(s):
+            re.sub(r'<[^>]*?>', '',
+                unicode(s).strip())
+
         pdfdoc = FancyDocTemplate(
             output,
             pageTemplates=[FP],
             showBoundary=0,
             pagesize=self.styles.ps,
-            title=self.doc_title,
+            title=self.doc_title_clean,
             author=self.doc_author,
             pageCompression=compressed)
+        pdfdoc.client =self
+
+        if getattr(self, 'mustMultiBuild', False):
+            # Force a multibuild pass
+            if not isinstance(elements[-1],UnhappyOnce):
+                log.info ('Forcing second pass so Total pages work')
+                elements.append(UnhappyOnce())
         while True:
             try:
                 log.info("Starting build")
-                pdfdoc.multiBuild(elements)
-                #from pudb import set_trace; set_trace()
                 # See if this *must* be multipass
+                pdfdoc.multiBuild(elements)
+                # Force a multibuild pass
+
+                # FIXME: since mustMultiBuild is set by the
+                # first pass in the case of ###Total###, then we
+                # make a new forced two-pass build. This is broken.
+                # conceptually.
+                
                 if getattr(self, 'mustMultiBuild', False):
+                    # Force a multibuild pass
                     if not isinstance(elements[-1],UnhappyOnce):
                         log.info ('Forcing second pass so Total pages work')
                         elements.append(UnhappyOnce())
                         continue
+                ## Rearrange footnotes if needed
+                if self.real_footnotes:
+                    newStory=[]
+                    fnPile=[]
+                    for e in elements:
+                        if getattr(e,'isFootnote',False):
+                            # Add it to the pile
+                            #if not isinstance (e, MySpacer):
+                            fnPile.append(e)
+                        elif getattr(e, '_atTop', False) or \
+                            isinstance (e, (UnhappyOnce, MyPageBreak)):
+                            if fnPile:
+                                newStory.append(Sinker(fnPile))
+                            newStory.append(e)
+                            fnPile=[]
+                        else:
+                            newStory.append(e)
+                    elements = newStory+fnPile
+                    for e in elements:
+                        if hasattr(e, '_postponed'):
+                            delattr(e,'_postponed')
+                    self.real_footnotes = False
+                    continue
+
+
+                
                 break
             except ValueError, v:
                 # FIXME: cross-document links come through here, which means
                 # an extra pass per cross-document reference. Which sucks.
-                if v.args and str(v.args[0]).startswith('format not resolved'):
-                    missing=str(v.args[0]).split(' ')[-1]
-                    log.error('Adding missing reference to %s and rebuilding. This is slow!'%missing)
-                    elements.append(Reference(missing))
-                    for e in elements:
-                        if hasattr(e,'_postponed'):
-                            delattr(e,'_postponed')
-                else:
-                    raise
+                #if v.args and str(v.args[0]).startswith('format not resolved'):
+                    #missing=str(v.args[0]).split(' ')[-1]
+                    #log.error('Adding missing reference to %s and rebuilding. This is slow!'%missing)
+                    #elements.append(Reference(missing))
+                    #for e in elements:
+                        #if hasattr(e,'_postponed'):
+                            #delattr(e,'_postponed')
+                #else:
+                    #raise
+                raise
+
         #doc = SimpleDocTemplate("phello.pdf")
         #doc.build(elements)
         for fn in self.to_unlink:
@@ -547,74 +727,10 @@ class RstToPdf(object):
                 pass
 
 
+from reportlab.platypus import doctemplate
+
 class FancyDocTemplate(BaseDocTemplate):
 
-    #def multiBuild(self, story,
-                   #filename=None,
-                   #canvasmaker=canvas.Canvas,
-                   #maxPasses = 10):
-        #"""Makes multiple passes until all indexing flowables
-        #are happy."""
-
-        #self._indexingFlowables = []
-        ##scan the story and keep a copy
-        #for thing in story:
-            #if thing.isIndexing():
-                #self._indexingFlowables.append(thing)
-
-        ##better fix for filename is a 'file' problem
-        #self._doSave = 0
-        #passes = 0
-        #mbe = []
-        #self._multiBuildEdits = mbe.append
-        #while 1:
-            #s=story[-202].style
-            #for n in dir(s):
-                #if not n.startswith('_'):
-                    #print n,eval('s.parent.%s'%n)
-            #print '----------------------'
-            #passes += 1
-            #log.info('Pass number %d'%passes)
-
-            #for fl in self._indexingFlowables:
-                #fl.beforeBuild()
-
-            ## work with a copy of the story, since it is consumed
-            #tempStory = story[:]
-            #self.build(tempStory, filename, canvasmaker)
-            ##self.notify('debug',None)
-
-            #for fl in self._indexingFlowables:
-                #fl.afterBuild()
-
-            #happy = self._allSatisfied()
-
-            #if happy:
-                #self._doSave = 0
-                #self.canv.save()
-                #break
-            #else:
-                #self.canv.save()
-                #f=open('pass-%d.pdf'%passes,'wb')
-                #f.seek(0)
-                #f.truncate()
-                #f.write(self.filename.getvalue())
-                #self.filename = StringIO()
-            #if passes > maxPasses:
-                ## Don't fail, just say that the indexes may be wrong
-                #log.error("Index entries not resolved after %d passes" % maxPasses)
-                #break
-
-
-            ##work through any edits
-            #while mbe:
-                #e = mbe.pop(0)
-                #e[0](*e[1:])
-
-        #del self._multiBuildEdits
-        #if verbose: print 'saved'
-
-
     def afterFlowable(self, flowable):
 
         if isinstance(flowable, Heading):
@@ -625,6 +741,77 @@ class FancyDocTemplate(BaseDocTemplate):
             pagenum = setPageCounter()
             self.notify('TOCEntry', (level, text, pagenum, parent_id, node))
 
+
+    def handle_flowable(self,flowables):
+        '''try to handle one flowable from the front of list flowables.'''
+
+        # this method is copied from reportlab
+
+        #allow document a chance to look at, modify or ignore
+        #the object(s) about to be processed
+        self.filterFlowables(flowables)
+
+        self.handle_breakBefore(flowables)
+        self.handle_keepWithNext(flowables)
+        f = flowables[0]
+        del flowables[0]
+        if f is None:
+            return
+
+        if isinstance(f,PageBreak):
+            if isinstance(f,SlowPageBreak):
+                self.handle_pageBreak(slow=1)
+            else:
+                self.handle_pageBreak()
+            self.afterFlowable(f)
+        elif isinstance(f,ActionFlowable):
+            f.apply(self)
+            self.afterFlowable(f)
+        else:
+            frame = self.frame
+            canv = self.canv
+            #try to fit it then draw it
+            if frame.add(f, canv, trySplit=self.allowSplitting):
+                if not isinstance(f,FrameActionFlowable):
+                    self._curPageFlowableCount += 1
+                    self.afterFlowable(f)
+                doctemplate._addGeneratedContent(flowables,frame)
+            else:
+                if self.allowSplitting:
+                    # see if this is a splittable thing
+                    S = frame.split(f,canv)
+                    n = len(S)
+                else:
+                    n = 0
+                if n:
+                    if not isinstance(S[0],(PageBreak,SlowPageBreak,ActionFlowable)):
+                        if frame.add(S[0], canv, trySplit=0):
+                            self._curPageFlowableCount += 1
+                            self.afterFlowable(S[0])
+                            doctemplate._addGeneratedContent(flowables,frame)
+                        else:
+                            ident = "Splitting error(n==%d) on page %d in\n%s" % (n,self.page,self._fIdent(f,60,frame))
+                            #leave to keep apart from the raise
+                            raise LayoutError(ident)
+                        del S[0]
+                    for i,f in enumerate(S):
+                        flowables.insert(i,f)   # put split flowables back on the list
+                else:
+                    if hasattr(f,'_postponed') and f._postponed > 4:
+                        ident = "Flowable %s%s too large on page %d in frame %r%s of template %r" % \
+                                (self._fIdent(f,60,frame),doctemplate._fSizeString(f),self.page, self.frame.id,
+                                        self.frame._aSpaceString(), self.pageTemplate.id)
+                        #leave to keep apart from the raise
+                        raise LayoutError(ident)
+                    # this ought to be cleared when they are finally drawn!
+                    f._postponed = 1
+                    mbe = getattr(self,'_multiBuildEdits',None)
+                    if mbe:
+                        mbe((delattr,f,'_postponed'))
+                    flowables.insert(0,f)           # put the flowable back
+                    self.handle_frameEnd()
+
+
 _counter=0
 _counterStyle='arabic'
 
@@ -678,9 +865,7 @@ class UnhappyOnce(IndexingFlowable):
     def isSatisfied(self):
         if self._unhappy:
             self._unhappy= False
-            print 'UNHAPPY'
             return False
-        print 'HAPPY'
         return True
         
     def draw(self):
@@ -709,6 +894,8 @@ class HeaderOrFooter(object):
         if showloc:
             if not items:
                 items = pageobj.template.get(self.defaultloc)
+                if items:
+                    items = self.client.gen_elements(publish_secondary_doctree(items, self.client.doctree, None))
             if items:
                 if isinstance(items, list):
                     items = items[:]
@@ -803,8 +990,8 @@ class FancyPage(PageTemplate):
             Calculates the image one time, and caches
             it for reuse on every page in the template.
 
-            Currently, reduces the image to fit on the page
-            and then centers it on the page.
+            How the background is drawn depends on the
+            --fit-background-mode option.
 
             If desired, we could add code to push it around
             on the page, using stylesheets to align and/or
@@ -819,16 +1006,26 @@ class FancyPage(PageTemplate):
                 log.error("Missing %s image file: %s", which, uri)
                 return
             try:
-                w, h, kind = MyImage.size_for_node(dict(uri=uri), self.client)
+                w, h, kind = MyImage.size_for_node(dict(uri=uri, ), self.client)
             except ValueError: 
                 # Broken image, return arbitrary stuff
                 uri=missing
                 w, h, kind = 100, 100, 'direct'
                 
             pw, ph = self.styles.pw, self.styles.ph
-            scale = min(1.0, 1.0 * pw / w, 1.0 * ph / h)
-            sw, sh = w * scale, h * scale
-            x, y = (pw - sw) / 2.0, (ph - sh) / 2.0
+            if self.client.background_fit_mode == 'center':
+                scale = min(1.0, 1.0 * pw / w, 1.0 * ph / h)
+                sw, sh = w * scale, h * scale
+                x, y = (pw - sw) / 2.0, (ph - sh) / 2.0
+            elif self.client.background_fit_mode == 'scale':
+                x, y = 0, 0
+                sw, sh = pw, ph
+            else:
+                log.error('Unknown background fit mode: %s'% self.client.background_fit_mode)
+                # Do scale anyway
+                x, y = 0, 0
+                sw, sh = pw, ph
+                
             bg = MyImage(uri, sw, sh, client=self.client)
             self.image_cache[uri] = info = bg, x, y
         bg, x, y = info
@@ -967,12 +1164,12 @@ def parse_commandline():
         help='A list of folders to search for fonts,'\
              ' separated using "%s". Default="%s"'%(os.pathsep, def_fontpath))
 
-    def_baseurl = urlunparse(['file',os.getcwd(),'','','',''])
+    def_baseurl = urlunparse(['file',os.getcwd()+os.sep,'','','',''])
     parser.add_option('--baseurl', dest='baseurl', metavar='URL',
         default=def_baseurl,
         help='The base URL for relative URLs. Default="%s"'%def_baseurl)
 
-    def_lang = config.getValue("general", "language", None)
+    def_lang = config.getValue("general", "language", 'en')
     parser.add_option('-l', '--language', metavar='LANG',
         default=def_lang, dest='language',
         help='Language to be used for hyphenation and '\
@@ -1003,6 +1200,13 @@ def parse_commandline():
         help='What todo when a literal is too wide. One of error,'\
         ' overflow,shrink,truncate. Default="%s"'%def_fit)
 
+    def_fit_background = config.getValue("general", "background_fit_mode",
+    "center")
+    parser.add_option('--fit-background-mode', metavar='MODE',
+        default=def_fit_background, dest='background_fit_mode',
+        help='How to fit the background image to the page.'\
+        ' One of scale or center. Default="%s"'%def_fit_background)
+
     parser.add_option('--inline-links', action="store_true",
     dest='inlinelinks', default=False,
         help='shows target between parenthesis instead of active link')
@@ -1041,6 +1245,13 @@ def parse_commandline():
         help='Show footnotes inline.'\
         ' Default=%s' % str(not def_inline_footnotes))
 
+    def_real_footnotes = config.getValue("general",
+        "real_footnotes", False)
+    parser.add_option('--real-footnotes', action='store_true',
+        dest='real_footnotes', default=def_real_footnotes,
+        help='Show footnotes at the bottom of the page where they are defined.'\
+        ' Default=%s' % str(def_real_footnotes))
+
     def_dpi = config.getValue("general", "default_dpi", 300)
     parser.add_option('--default-dpi', dest='def_dpi', metavar='NUMBER',
         default=def_dpi,
@@ -1089,6 +1300,21 @@ def parse_commandline():
         help="Add a helper extension module to this invocation of rst2pdf "
              "(module must end in .py and be on the python path)")
 
+    def_cover = config.getValue("general", "custom_cover", 'cover.tmpl')
+    parser.add_option('--custom-cover', dest='custom_cover',
+        metavar='FILE', default= def_cover,
+        help='Template file used for the cover page. Default: %s'%def_cover)
+
+    def_floating_images = config.getValue("general", "floating_images", False)
+    parser.add_option('--use-floating-images', action='store_true', default=def_floating_images,
+        help='Makes images with :aling: attribute work more like in rst2html. Default: %s'%def_floating_images,
+        dest='floating_images')
+
+    def_numbered_links = config.getValue("general", "numbered_links", False)
+    parser.add_option('--use-numbered-links', action='store_true', default=def_numbered_links,
+        help='When using numbered sections, adds the numbers to all links referring to the section headers. Default: %s'%def_numbered_links,
+        dest='numbered_links')
+
     return parser
 
 def main(args=None):
@@ -1136,7 +1362,11 @@ def main(args=None):
     else:
         filename = args[0]
         options.basedir=os.path.dirname(os.path.abspath(filename))
-        infile = open(filename)
+        try:
+            infile = open(filename)
+        except IOError, e:
+            log.error(e)
+            sys.exit(1)
     options.infile = infile
 
     if options.output:
@@ -1181,6 +1411,9 @@ def main(args=None):
         spath = options.stylepath.split(os.pathsep)
     options.stylepath = spath
 
+    if options.real_footnotes:
+        options.inline_footnotes = True
+
     if reportlab.Version < '2.3':
         log.warning('You are using Reportlab version %s.'\
             ' The suggested version '\
@@ -1200,23 +1433,30 @@ def main(args=None):
         breaklevel=int(options.breaklevel),
         baseurl=options.baseurl,
         fit_mode=options.fit_mode,
+        background_fit_mode = options.background_fit_mode,
         smarty=str(options.smarty),
         font_path=options.fpath,
         style_path=options.stylepath,
         repeat_table_rows=options.repeattablerows,
         footnote_backlinks=options.footnote_backlinks,
         inline_footnotes=options.inline_footnotes,
+        real_footnotes=options.real_footnotes,
         def_dpi=int(options.def_dpi),
         basedir=options.basedir,
         show_frame=options.show_frame,
         splittables=options.splittables,
         blank_first_page=options.blank_first_page,
-        breakside=options.breakside
+        breakside=options.breakside,
+        custom_cover=options.custom_cover,
+        floating_images=options.floating_images,
+        numbered_links=options.numbered_links,
         ).createPdf(text=options.infile.read(),
                     source_path=options.infile.name,
                     output=options.outfile,
                     compressed=options.compressed)
 
+# Ugly hack that fixes Issue 335
+reportlab.lib.utils.ImageReader.__deepcopy__ = lambda self,*x: copy(self)
 
 def patch_digester():
     ''' Patch digester so that we can get the same results when image
@@ -1360,5 +1600,32 @@ def monkeypatch():
 
 monkeypatch()
 
+def publish_secondary_doctree(text, main_tree, source_path):
+
+    # This is a hack so the text substitutions defined
+    # in the document are available when we process the cover
+    # page. See Issue 322
+    dt = main_tree
+    # Add substitutions  from the main doctree
+    class addSubsts(Transform):
+        default_priority = 219
+
+        def apply(self):
+            self.document.substitution_defs.update(dt.substitution_defs)
+            self.document.substitution_names.update(dt.substitution_names)
+
+    # Use an own reader to modify transformations done.
+    class Reader(standalone.Reader):
+
+        def get_transforms(self):
+            default = standalone.Reader.get_transforms(self)
+            return (default + [ addSubsts, ])
+
+    # End of Issue 322 hack
+
+    return docutils.core.publish_doctree(text,
+        reader = Reader(), source_path=source_path)
+
+
 if __name__ == "__main__":
     main(sys.argv[1:])
