@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 # See LICENSE.txt for licensing terms
-#$HeadURL: https://rst2pdf.googlecode.com/svn/branches/0.14/rst2pdf/flowables.py $
-#$LastChangedDate: 2010-03-25 23:12:19 -0300 (Thu, 25 Mar 2010) $
-#$LastChangedRevision: 2052 $
+#$HeadURL: https://rst2pdf.googlecode.com/svn/tags/0.16/rst2pdf/flowables.py $
+#$LastChangedDate: 2010-10-06 17:17:48 -0300 (Wed, 06 Oct 2010) $
+#$LastChangedRevision: 2393 $
 
 __docformat__ = 'reStructuredText'
 
@@ -26,6 +26,39 @@ from log import log
 import re
 from xml.sax.saxutils import unescape, escape
 
+class XXPreformatted(XPreformatted):
+    """An extended XPreformattedFit"""
+    def __init__(self, *args, **kwargs):
+        XPreformatted.__init__(self, *args, **kwargs)
+
+    def split (self, aW, aH):
+
+        # Figure out a nice range of splits
+        #
+        # Assume we would prefer 5 lines (at least) on
+        # a splitted flowable before a break, and 4 on
+        # the last flowable after a break.
+        # So, the minimum wrap height for a fragment
+        # will be 5*leading
+
+        rW, rH = self.wrap(aW, aH)
+        if rH > aH:
+
+            minH1=getattr(self.style, 'allowOrphans', 5)*self.style.leading
+            minH2=getattr(self.style, 'allowWidows', 4)*self.style.leading
+
+            # If there's no way to fid a decent fragment,
+            # refuse to split
+            if aH < minH1:
+                return []
+
+            # Now, don't split too close to the end either
+            pw, ph = self.wrap(aW, aH)
+            if ph - aH < minH2:
+                aH = ph - minH2
+            
+        return XPreformatted.split(self, aW, aH)
+
 class MyIndenter(Indenter):
     """An indenter that has a width, because otherwise you get crashes
     if added inside tables"""
@@ -152,7 +185,7 @@ class OddEven(Flowable):
         """Makes no sense to split this..."""
         return []
 
-class DelayedTable(Flowable):
+class DelayedTable(Table):
     """A flowable that inserts a table for which it has the data.
 
     Needed so column widths can be determined after we know on what frame
@@ -169,6 +202,18 @@ class DelayedTable(Flowable):
         self.hAlign = TA_CENTER
         self.splitByRow=splitByRow
 
+        ## Try to look more like a Table
+        #self._ncols = 2
+        #self._nosplitCmds= []
+        #self._nrows= 1
+        #self._rowHeights= [None]
+        #self._spanCmds= []
+        #self.ident= None
+        #self.repeatCols= 0
+        #self.repeatRows= 0
+        #self.splitByRow= 1
+        #self.vAlign= 'MIDDLE'
+        
     def wrap(self, w, h):
         # Create the table, with the widths from colWidths reinterpreted
         # if needed as percentages of frame/cell/whatever width w is.
@@ -189,6 +234,8 @@ class DelayedTable(Flowable):
 
     def split(self, w, h):
         if self.splitByRow:
+            if not self.t:
+                self.wrap(w,h)
             return self.t.split(w, h)
         else:
             return []
@@ -197,7 +244,6 @@ class DelayedTable(Flowable):
         self.t.drawOn(canvas, x, y, _sW)
         
     def identity(self, maxLen=None):
-        print self.data
         return "<%s at %s%s%s> containing: %s" % (self.__class__.__name__,
             hex(id(self)), self._frameName(),
             getattr(self, 'name', '')
@@ -328,6 +374,13 @@ class SplitTable(DelayedTable):
             return DelayedTable.split(self,w,h)
 
 
+class MySpacer(Spacer):
+    def wrap (self, aW, aH):
+        w, h = Spacer.wrap(self, aW, aH)
+        self.height = min(aH, h)
+        return w, self.height
+
+    
 
 class MyPageBreak(FrameActionFlowable):
 
@@ -365,7 +418,7 @@ class MyPageBreak(FrameActionFlowable):
                 if not frame._atTop:
                     # Blank pages get no heading or footer
                     frame._generated_content.append(SetNextTemplate(self.templateName))
-                    frame._generated_content.append(SetNextTemplate('coverPage'))
+                    frame._generated_content.append(SetNextTemplate('emptyPage'))
                     frame._generated_content.append(PageBreak())
                     frame._generated_content.append(ResetNextTemplate())
                     frame._generated_content.append(PageBreak())
@@ -381,7 +434,7 @@ class MyPageBreak(FrameActionFlowable):
                 if not frame._atTop:
                     # Blank pages get no heading or footer
                     frame._generated_content.append(SetNextTemplate(self.templateName))
-                    frame._generated_content.append(SetNextTemplate('coverPage'))
+                    frame._generated_content.append(SetNextTemplate('emptyPage'))
                     frame._generated_content.append(PageBreak())
                     frame._generated_content.append(ResetNextTemplate())
                     frame._generated_content.append(PageBreak())
@@ -426,6 +479,8 @@ class ResetNextTemplate(Flowable):
         self.canv.templateName, self.canv.oldTemplateName = \
             self.canv.oldTemplateName, self.canv.templateName
 
+    def wrap(self, aW, aH):
+        return 0,0
 
 class Transition(Flowable):
     """Wrap canvas.setPageTransition.
@@ -518,7 +573,7 @@ class FrameCutter(FrameActionFlowable):
     def frameAction(self, frame):
         idx = frame.container.frames.index(frame)
         if self.floatLeft:
-            # Don´ t bother inserting a silly thin frame
+            # Don't bother inserting a silly thin frame
             if self.width-self.padding > 30:
                 f1 = SmartFrame(frame.container,
                     frame._x1 + self.dx - 2*self.padding,
@@ -540,7 +595,7 @@ class FrameCutter(FrameActionFlowable):
                         frame._height - self.f.height - 3*self.padding,
                         topPadding=0))
         else:
-            # Don´ t bother inserting a silly thin frame
+            # Don't bother inserting a silly thin frame
             if self.width-self.padding > 30:
                 f1 = SmartFrame(frame.container,
                     frame._x1 - self.width,
@@ -699,11 +754,9 @@ class BoundByWidth(Flowable):
         _sW = 0
         scale = self.scale
         content = None
-        aW = None
         #, canv, x, y, _sW=0, scale=1.0, content=None, aW=None):
         pS = 0
-        if aW is None:
-            aW = self.width
+        aW = self.width
         aW = scale*(aW + _sW)
         if content is None:
             content = self.content
@@ -899,7 +952,7 @@ class MyTableOfContents(TableOfContents):
             
         if rlabel in self.refids and islocal(node):
             self.addEntry(level, text, pageNum)
-            self.refid_lut[(level, text)] = label
+            self.refid_lut[(level, text, pageNum)] = label
 
     def wrap(self, availWidth, availHeight):
         """Adds hyperlink to toc entry."""
@@ -932,7 +985,7 @@ class MyTableOfContents(TableOfContents):
                 leftColStyle=self.getLevelStyle(left_col_level)
             else: # For ReportLab <= 2.3
                 leftColStyle = self.levelStyles[left_col_level]
-            label = self.refid_lut.get((level, text), None)
+            label = self.refid_lut.get((level, text, pageNum), None)
             if label:
                 pre = u'<a href="%s" color="%s">' % (label, self.linkColor)
                 post = u'</a>'
