@@ -8,6 +8,8 @@ from copy import copy
 from reportlab.platypus.flowables import Image, Flowable
 from log import log, nodeid
 from reportlab.lib.units import *
+import glob
+import urllib
 
 from opt_imports import LazyImports
 
@@ -72,9 +74,19 @@ class MyImage (Flowable):
                  kind='direct', mask="auto", lazy=1, client=None):
         # Client is mandatory.  Perhaps move it farther up if we refactor
         assert client is not None
-        srcinfo = client, filename
         self.__kind=kind
+
+        if filename.split("://")[0].lower() in ('http','ftp','https'):
+            try:
+                filename2, _ = urllib.urlretrieve(filename)
+                if filename != filename2:
+                    client.to_unlink.append(filename2)
+                    filename = filename2
+            except IOError:
+                filename = missing
         self.filename, self._backend=self.get_backend(filename, client)
+        srcinfo = client, self.filename
+        
         if kind == 'percentage_of_container':
             self.image=self._backend(self.filename, width, height,
                 'direct', mask, lazy, srcinfo)
@@ -181,13 +193,47 @@ class MyImage (Flowable):
         where fname is the filename to be used (could be the same as
         filename, or something different if the image had to be converted
         or is missing), and backend is an Image class that can handle 
-        fname.'''
+        fname.
+
+
+        If uri ensd with '.*' then the returned filename will be the best
+        quality supported at the moment.
+
+        That means:  PDF > SVG > anything else
+
+        '''
 
         backend = defaultimage
 
+
         # Extract all the information from the URI
         filename, extension, options = self.split_uri(uri)
 
+        if '*' in filename:
+            preferred=['gif','jpg','png']
+            if SVGImage.available():
+                preferred.append('svg')
+            preferred.append('pdf')
+
+            # Find out what images are available
+            available = glob.glob(filename)
+            cfn=available[0]
+            cv=-10
+            for fn in available:
+                ext=fn.split('.')[-1]
+                if ext in preferred:
+                    v=preferred.index(ext)
+                else:
+                    v=-1
+                if v > cv:
+                    cv=v
+                    cfn=fn
+            # cfn should have our favourite type of
+            # those available
+            filename = cfn
+            extension = cfn.split('.')[-1]
+            uri = filename
+
         # If the image doesn't exist, we use a 'missing' image
         if not os.path.exists(filename):
             log.error("Missing image file: %s",filename)
@@ -218,7 +264,7 @@ class MyImage (Flowable):
                 filename=self.raster(filename, client)
             else:
                 log.warning("Minimal PDF image support "\
-                    "requires PythonMagick [%s]", filename)
+                    "requires PythonMagick or the vectorpdf extension [%s]", filename)
                 filename = missing
         elif extension != 'jpg' and not LazyImports.PILImage:
             if LazyImports.PMImage:
@@ -237,9 +283,14 @@ class MyImage (Flowable):
         in the PDF document, and what 'kind' of size that is. 
         That involves lots of guesswork'''
 
-        uri = os.path.join(client.basedir,str(node.get("uri")))
+        uri = str(node.get("uri"))
+        if uri.split("://")[0].lower() not in ('http','ftp','https'):
+            uri = os.path.join(client.basedir,uri)
+        else:
+            uri, _ = urllib.urlretrieve(uri)
+            client.to_unlink.append(uri)
+            
         srcinfo = client, uri
-        
         # Extract all the information from the URI
         imgname, extension, options = self.split_uri(uri)
 
@@ -313,9 +364,16 @@ class MyImage (Flowable):
                 ydpi=density.height()*2.54
                 keeptrying = False
             if keeptrying:
-                log.error("The image (%s, %s) is broken or in an unknown format"
-                            , imgname, nodeid(node))
-                raise ValueError
+                if extension not in ['jpg', 'jpeg']:
+                    log.error("The image (%s, %s) is broken or in an unknown format"
+                                , imgname, nodeid(node))
+                    raise ValueError
+                else:
+                    # Can be handled by reportlab
+                    log.warning("Can't figure out size of the image (%s, %s). Install PIL for better results."
+                                , imgname, nodeid(node))
+                    iw = 1000
+                    ih = 1000
 
         # Try to get the print resolution from the image itself via PIL.
         # If it fails, assume a DPI of 300, which is pretty much made up,
@@ -331,7 +389,7 @@ class MyImage (Flowable):
         h = node.get('height')
         if h is None and w is None: # Nothing specified
             # Guess from iw, ih
-            log.warning("Using image %s without specifying size."
+            log.debug("Using image %s without specifying size."
                 "Calculating based on image size at %ddpi [%s]",
                 imgname, xdpi, nodeid(node))
             w = iw*inch/xdpi
