@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 # See LICENSE.txt for licensing terms
-#$HeadURL: https://rst2pdf.googlecode.com/svn/branches/0.14/rst2pdf/styles.py $
-#$LastChangedDate: 2010-03-20 12:04:22 -0300 (Sat, 20 Mar 2010) $
-#$LastChangedRevision: 1967 $
+#$HeadURL: https://rst2pdf.googlecode.com/svn/tags/0.16/rst2pdf/styles.py $
+#$LastChangedDate: 2010-09-09 22:21:45 -0300 (Thu, 09 Sep 2010) $
+#$LastChangedRevision: 2336 $
 
 import os
 import sys
@@ -23,6 +23,7 @@ from reportlab.lib.styles import *
 from reportlab.lib.enums import *
 from reportlab.pdfbase import pdfmetrics
 import reportlab.lib.pagesizes as pagesizes
+import reportlab.rl_config
 
 from rst2pdf.rson import loads as rson_loads
 
@@ -149,6 +150,11 @@ class StyleSheet(object):
                         self.psname = pgs
                         if 'width' in self.page: del(self.page['width'])
                         if 'height' in self.page: del(self.page['height'])
+                    elif pgs.endswith('-LANDSCAPE'):
+                        self.psname = pgs.split('-')[0]
+                        self.ps = list(pagesizes.landscape(pagesizes.__dict__[self.psname]))
+                        if 'width' in self.page: del(self.page['width'])
+                        if 'height' in self.page: del(self.page['height'])
                     else:
                         log.critical('Unknown page size %s in stylesheet %s'%\
                             (page['size'], ssname))
@@ -372,6 +378,7 @@ class StyleSheet(object):
                             log.error("Unknown font: \"%s\","
                                       "replacing with Helvetica", style[key])
                             style[key] = "Helvetica"
+                            
         #log.info('FontList: %s'%self.embedded)
         #log.info('FontAlias: %s'%self.fontsAlias)
         # Get styles from all stylesheets in order
@@ -433,8 +440,28 @@ class StyleSheet(object):
                     self.stylesheet[skey] = sdict
                     self.styles.append(sdict)
 
+        # If the stylesheet has a style name docutils won't reach
+        # make a copy with a sanitized name.
+        # This may make name collisions possible but that should be
+        # rare (who would have custom_name and custom-name in the
+        # same stylesheet? ;-)
+        # Issue 339
+        
+        styles2=[]
+        for s in self.styles:
+            if not re.match("^[a-z](-?[a-z0-9]+)*$", s['name']):
+                s2 = copy(s)
+                s2['name'] = docutils.nodes.make_id(s['name'])
+                log.warning('%s is an invalid docutils class name, adding alias %s'%(s['name'], s2['name']))
+                styles2.append(s2)
+        self.styles.extend(styles2)
+
         # And create  reportlabs stylesheet
         self.StyleSheet = StyleSheet1()
+        # Patch to make the code compatible with reportlab from SVN 2.4+ and
+        # 2.4
+        if not hasattr(self.StyleSheet, 'has_key'):
+            self.StyleSheet.__class__.has_key = lambda s, k : k in s
         for s in self.styles:
             if 'parent' in s:
                 if s['parent'] is None:
@@ -456,6 +483,7 @@ class StyleSheet(object):
             # Adjust fontsize units
             if 'fontSize' not in s:
                 s['fontSize'] = s['parent'].fontSize
+                s['trueFontSize']=None
                 hasFS = False
             elif 'parent' in s:
                 # This means you can set the fontSize to
@@ -463,6 +491,7 @@ class StyleSheet(object):
                 # relative to the parent style
                 s['fontSize'] = self.adjustUnits(s['fontSize'],
                                     s['parent'].fontSize)
+                s['trueFontSize']=s['fontSize']
             else:
                 # If s has no parent, it's base, which has
                 # an explicit point size by default and %
@@ -489,9 +518,24 @@ class StyleSheet(object):
                 s['borderPadding']=s['borderPadding'][0]
 
             self.StyleSheet.add(ParagraphStyle(**s))
+
+        
         self.emsize=self['base'].fontSize
+        # Make stdFont the basefont, for Issue 65
+        reportlab.rl_config.canvas_basefontname = self['base'].fontName
+        # Make stdFont the default font for table cell styles (Issue 65)
+        reportlab.platypus.tables.CellStyle1.fontname=self['base'].fontName
+        
 
     def __getitem__(self, key):
+
+        # This 'normalizes' the key.
+        # For example, if the key is todo_node (like sphinx uses), it will be
+        # converted to 'todo-node' which is a valid docutils class name.
+
+        if not re.match("^[a-z](-?[a-z0-9]+)*$", key):
+            key = docutils.nodes.make_id(key)
+
         if self.StyleSheet.has_key(key):
             return self.StyleSheet[key]
         else:
@@ -927,3 +971,4 @@ class CallableStyleSheet(str):
         return self
     def __call__(self):
         return rson_loads(self.value)
+
