@@ -4,12 +4,13 @@
 
 f1=$1
 f2=$2
+flag=0
 
-tmpdir=/tmp/comppdf-$$
+tmpdir=/tmp/comppdf-$$ 
 mkdir $tmpdir
-convert -density 300x300 $f1 $tmpdir/page.png
-convert -density 300x300 $f2 $tmpdir/bpage.png
-pushd $tmpdir
+convert -density 96x96 $f1 $tmpdir/page.png
+convert -density 96x96 $f2 $tmpdir/bpage.png
+pushd $tmpdir >/dev/null 2>&1
 
 for page in page*.png
 do
@@ -20,8 +21,8 @@ do
             true
     else
             echo "$page has ERRORs, see $tmpdir/diff$page"
+	    flag=1
     fi    
 done
-popd
-
-convert $tmpdir/diff*png diff-$$.pdf
+popd >/dev/null 2>&1
+exit $flag
