@@ -1,19 +1,17 @@
 # -*- coding: utf-8 -*-
 # See LICENSE.txt for licensing terms
-#$HeadURL: https://rst2pdf.googlecode.com/svn/branches/0.14/rst2pdf/utils.py $
-#$LastChangedDate: 2009-11-03 03:15:32 -0300 (Tue, 03 Nov 2009) $
-#$LastChangedRevision: 1312 $
+#$HeadURL: https://rst2pdf.googlecode.com/svn/tags/0.16/rst2pdf/utils.py $
+#$LastChangedDate: 2010-09-19 16:25:49 -0300 (Sun, 19 Sep 2010) $
+#$LastChangedRevision: 2349 $
 
 import shlex
 
-from reportlab.platypus import Spacer
-
 from flowables import *
 import flowables
 from styles import adjustUnits
-from log import log
+from log import log, nodeid
 
-def parseRaw(data):
+def parseRaw(data, node):
     """Parse and process a simple DSL to handle creation of flowables.
 
     Supported (can add others on request):
@@ -29,6 +27,8 @@ def parseRaw(data):
         lexer = shlex.shlex(line)
         lexer.whitespace += ','
         tokens = list(lexer)
+        if not tokens:
+            continue # Empty line
         command = tokens[0]
         if command == 'PageBreak':
             if len(tokens) == 1:
@@ -46,14 +46,14 @@ def parseRaw(data):
             else:
                 elements.append(MyPageBreak(tokens[1],breakTo='odd'))
         elif command == 'Spacer':
-            elements.append(Spacer(adjustUnits(tokens[1]), 
+            elements.append(MySpacer(adjustUnits(tokens[1]), 
                 adjustUnits(tokens[2])))
         elif command == 'Transition':
             elements.append(Transition(*tokens[1:]))
         elif command == 'SetPageCounter':
             elements.append(flowables.PageCounter(*tokens[1:]))
         else:
-            log.error('Unknown command %s in raw pdf directive'%command)
+            log.error('Unknown command %s in raw pdf directive [%s]'%(command,nodeid(node)))
     return elements
 
 
